Package: scCustomize
Type: Package
Title: Custom Visualizations & Functions for Streamlined Analyses of
        Single Cell Sequencing
Description: Collection of functions created and/or curated to aid in the visualization and analysis of single-cell data using 'R'.  'scCustomize' aims to provide 1) Customized visualizations for aid in ease of use and to create more aesthetic and functional visuals. 2) Improve speed/reproducibility of common tasks/pieces of code in scRNA-seq analysis with a single or group of functions.  For citation please use: Marsh SE (2021) "Custom Visualizations & Functions for Streamlined Analyses of Single Cell Sequencing" <doi:10.5281/zenodo.5706430> RRID:SCR_024675.
Version: 3.3.0
Date: 2026-05-05
Authors@R: c(
    person(given = "Samuel", family = "Marsh", email = "sccustomize@gmail.com", role = c("aut", "cre"), comment = c(ORCID = "0000-0002-3012-6945")),
    person(given = "Ming", family = "Tang", role = c("ctb"), email = "tangming2005@gmail.com"),
    person(given = 'Velina', family = 'Kozareva', role = c('ctb')),
    person(given ="Lucas", family = "Graybuck", email = "lucasg@alleninstitute.org", role = c('ctb')),
    person(given ="Zoe", family = "Clarke", email = "zoe.clarke@utoronto.ca", role = c('ctb'), comment = c(ORCID = "0000-0003-0336-8049"))
    )
URL: https://github.com/samuel-marsh/scCustomize,
        https://samuel-marsh.github.io/scCustomize/,
        https://doi.org/10.5281/zenodo.5706431
BugReports: https://github.com/samuel-marsh/scCustomize/issues
Depends: R (>= 4.0.0), Seurat (>= 4.3.0.1)
Imports: circlize, cli (>= 3.2.0), cowplot, data.table, dplyr, forcats,
        ggbeeswarm, ggplot2, ggprism, ggrastr, ggrepel, glue,
        grDevices, grid, janitor, lifecycle, magrittr, Matrix (>=
        1.5.0), mcprogress, methods, paletteer, patchwork, pbapply,
        purrr, rlang (>= 1.1.3), scales, scattermore (>= 1.2),
        SeuratObject (>= 5.0.0), stats, stringi, stringr, tibble, tidyr
Suggests: BiocFileCache, ComplexHeatmap, dittoSeq, DropletUtils,
        ggpubr, hdf5r, knitr, Nebulosa, remotes, reticulate, rliger,
        rmarkdown, scuttle, tidyselect, qs2, viridis
License: GPL (>= 3)
Encoding: UTF-8
LazyData: true
RoxygenNote: 7.3.3
NeedsCompilation: no
Packaged: 2026-05-08 12:37:49 UTC; samuelmarsh
Author: Samuel Marsh [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-3012-6945>),
  Ming Tang [ctb],
  Velina Kozareva [ctb],
  Lucas Graybuck [ctb],
  Zoe Clarke [ctb] (ORCID: <https://orcid.org/0000-0003-0336-8049>)
Maintainer: Samuel Marsh <sccustomize@gmail.com>
Repository: CRAN
Date/Publication: 2026-05-08 13:00:02 UTC
Built: R 4.6.0; ; 2026-05-08 16:03:56 UTC; unix
