# Generated by roxygen2: do not edit by hand

S3method("Idents<-",liger)
S3method(Add_Cell_Complexity,Seurat)
S3method(Add_Cell_Complexity,liger)
S3method(Add_Cell_QC_Metrics,Seurat)
S3method(Add_Cell_QC_Metrics,liger)
S3method(Add_Hemo,Seurat)
S3method(Add_Hemo,liger)
S3method(Add_MALAT1_Threshold,Seurat)
S3method(Add_Mito_Ribo,Seurat)
S3method(Add_Mito_Ribo,liger)
S3method(Add_Top_Gene_Pct,Seurat)
S3method(Add_Top_Gene_Pct,liger)
S3method(Cells,liger)
S3method(Embeddings,liger)
S3method(Features,liger)
S3method(Fetch_Meta,Seurat)
S3method(Fetch_Meta,liger)
S3method(Idents,liger)
S3method(Rename_Clusters,Seurat)
S3method(Rename_Clusters,liger)
S3method(Top_Genes_Factor,Seurat)
S3method(Top_Genes_Factor,liger)
S3method(WhichCells,liger)
S3method(as.LIGER,Seurat)
S3method(as.LIGER,list)
S3method(as.Seurat,liger)
S3method(as.anndata,Seurat)
S3method(as.anndata,liger)
export("Idents<-")
export(Add_Alt_Feature_ID)
export(Add_CellBender_Diff)
export(Add_Cell_Complexity)
export(Add_Cell_QC_Metrics)
export(Add_Hemo)
export(Add_MALAT1_Threshold)
export(Add_Mito_Ribo)
export(Add_Pct_Diff)
export(Add_Sample_Meta)
export(Add_Top_Gene_Pct)
export(Barcode_Plot)
export(Blank_Theme)
export(Case_Check)
export(CellBender_Diff_Plot)
export(CellBender_Feature_Diff)
export(Cell_Highlight_Plot)
export(Cells)
export(Cells_by_Identities_LIGER)
export(Cells_per_Sample)
export(Change_Delim_All)
export(Change_Delim_Prefix)
export(Change_Delim_Suffix)
export(CheckMatrix_scCustom)
export(Cluster_Highlight_Plot)
export(Cluster_Stats_All_Samples)
export(Clustered_DotPlot)
export(ColorBlind_Pal)
export(Convert_Assay)
export(Copy_From_GCP)
export(Copy_To_GCP)
export(Create_10X_H5)
export(Create_CellBender_Merged_Seurat)
export(Create_Cluster_Annotation_File)
export(Dark2_Pal)
export(Dataset_Size_LIGER)
export(DimPlot_All_Samples)
export(DimPlot_LIGER)
export(DimPlot_scCustom)
export(DiscretePalette_scCustomize)
export(DotPlot_scCustom)
export(ElbowPlot_scCustom)
export(Embeddings)
export(Extract_Modality)
export(Extract_Sample_Meta)
export(Extract_Top_Markers)
export(Factor_Cor_Plot)
export(FeaturePlot_DualAssay)
export(FeaturePlot_scCustom)
export(FeatureScatter_scCustom)
export(Feature_Present)
export(Features)
export(Fetch_Meta)
export(Find_Factor_Cor)
export(Get_Reference_LIGER)
export(Hue_Pal)
export(Idents)
export(Iterate_Barcode_Rank_Plot)
export(Iterate_Cluster_Highlight_Plot)
export(Iterate_DimPlot_bySample)
export(Iterate_FeaturePlot_scCustom)
export(Iterate_Meta_Highlight_Plot)
export(Iterate_PC_Loading_Plots)
export(Iterate_Plot_Density_Custom)
export(Iterate_Plot_Density_Joint)
export(Iterate_VlnPlot_scCustom)
export(JCO_Four)
export(MAD_Stats)
export(Map_New_Meta)
export(Median_Stats)
export(Merge_Seurat_List)
export(Merge_Sparse_Data_All)
export(Merge_Sparse_Multimodal_All)
export(Meta_Highlight_Plot)
export(Meta_Numeric)
export(Meta_Present)
export(Meta_Remove_Seurat)
export(Move_Legend)
export(NavyAndOrange)
export(PC_Plotting)
export(PalettePlot)
export(Percent_Expressing)
export(Plot_Cells_per_Sample)
export(Plot_Density_Custom)
export(Plot_Density_Joint_Only)
export(Plot_Median_Genes)
export(Plot_Median_Mito)
export(Plot_Median_Other)
export(Plot_Median_UMIs)
export(Proportion_Plot)
export(Proportion_Plot_per_Sample)
export(Pull_Cluster_Annotation)
export(Pull_Directory_List)
export(QC_Histogram)
export(QC_Plot_GenevsFeature)
export(QC_Plot_UMIvsFeature)
export(QC_Plot_UMIvsGene)
export(QC_Plots_Combined_Vln)
export(QC_Plots_Complexity)
export(QC_Plots_Feature)
export(QC_Plots_Genes)
export(QC_Plots_Mito)
export(QC_Plots_UMIs)
export(Random_Cells_Downsample)
export(ReFilter_SeuratObject)
export(Read10X_GEO)
export(Read10X_Multi_Directory)
export(Read10X_h5_GEO)
export(Read10X_h5_Multi_Directory)
export(Read_Add_cNMF)
export(Read_CellBender_h5_Mat)
export(Read_CellBender_h5_Multi_Directory)
export(Read_CellBender_h5_Multi_File)
export(Read_GEO_Delim)
export(Read_Metrics_10X)
export(Read_Metrics_CellBender)
export(Read_Velocity)
export(Reduction_Loading_Present)
export(Rename_Clusters)
export(Replace_Suffix)
export(Seq_QC_Plot_Alignment_Combined)
export(Seq_QC_Plot_Antisense)
export(Seq_QC_Plot_Basic_Combined)
export(Seq_QC_Plot_Exonic)
export(Seq_QC_Plot_Genes)
export(Seq_QC_Plot_Genome)
export(Seq_QC_Plot_Intergenic)
export(Seq_QC_Plot_Intronic)
export(Seq_QC_Plot_Number_Cells)
export(Seq_QC_Plot_Reads_in_Cells)
export(Seq_QC_Plot_Reads_per_Cell)
export(Seq_QC_Plot_Saturation)
export(Seq_QC_Plot_Total_Genes)
export(Seq_QC_Plot_Transcriptome)
export(Seq_QC_Plot_UMIs)
export(Setup_scRNAseq_Project)
export(Single_Color_Palette)
export(SpatialDimPlot_scCustom)
export(Split_Layers)
export(Split_Vector)
export(Stacked_VlnPlot)
export(Store_Misc_Info_Seurat)
export(Store_Palette_Seurat)
export(Subset_LIGER)
export(Top_Genes_Factor)
export(UnRotate_X)
export(Updated_HGNC_Symbols)
export(Updated_MGI_Symbols)
export(VariableFeaturePlot_scCustom)
export(Variable_Features_ALL_LIGER)
export(VlnPlot_scCustom)
export(WhichCells)
export(as.LIGER)
export(as.Seurat)
export(as.anndata)
export(exAM_Scoring)
export(plotFactors_scCustom)
export(scCustomize_Palette)
export(seq_zeros)
export(theme_ggprism_mod)
export(viridis_dark_high)
export(viridis_inferno_dark_high)
export(viridis_inferno_light_high)
export(viridis_light_high)
export(viridis_magma_dark_high)
export(viridis_magma_light_high)
export(viridis_plasma_dark_high)
export(viridis_plasma_light_high)
import(Matrix)
import(Seurat)
import(cli)
import(ggplot2)
import(ggrastr)
import(mcprogress)
import(parallel)
import(patchwork)
import(pbapply)
importFrom(Matrix,readMM)
importFrom(Matrix,rowSums)
importFrom(Seurat,AddMetaData)
importFrom(Seurat,DimPlot)
importFrom(Seurat,DotPlot)
importFrom(Seurat,FeaturePlot)
importFrom(Seurat,FeatureScatter)
importFrom(Seurat,PCHeatmap)
importFrom(Seurat,PercentageFeatureSet)
importFrom(Seurat,Read10X)
importFrom(Seurat,Read10X_h5)
importFrom(Seurat,VariableFeaturePlot)
importFrom(Seurat,VizDimLoadings)
importFrom(Seurat,VlnPlot)
importFrom(SeuratObject,"Idents<-")
importFrom(SeuratObject,Cells)
importFrom(SeuratObject,DefaultDimReduc)
importFrom(SeuratObject,Embeddings)
importFrom(SeuratObject,Features)
importFrom(SeuratObject,Idents)
importFrom(SeuratObject,JoinLayers)
importFrom(SeuratObject,LayerData)
importFrom(SeuratObject,Layers)
importFrom(SeuratObject,WhichCells)
importFrom(SeuratObject,as.Seurat)
importFrom(circlize,colorRamp2)
importFrom(cowplot,theme_cowplot)
importFrom(data.table,fread)
importFrom(dplyr,across)
importFrom(dplyr,all_of)
importFrom(dplyr,any_of)
importFrom(dplyr,arrange)
importFrom(dplyr,bind_rows)
importFrom(dplyr,contains)
importFrom(dplyr,desc)
importFrom(dplyr,filter)
importFrom(dplyr,group_by)
importFrom(dplyr,grouped_df)
importFrom(dplyr,intersect)
importFrom(dplyr,join_by)
importFrom(dplyr,left_join)
importFrom(dplyr,mutate)
importFrom(dplyr,n)
importFrom(dplyr,pull)
importFrom(dplyr,rename)
importFrom(dplyr,right_join)
importFrom(dplyr,select)
importFrom(dplyr,setdiff)
importFrom(dplyr,slice)
importFrom(dplyr,slice_max)
importFrom(dplyr,slice_min)
importFrom(dplyr,summarise)
importFrom(dplyr,summarize)
importFrom(dplyr,ungroup)
importFrom(dplyr,union)
importFrom(forcats,fct_relevel)
importFrom(forcats,fct_rev)
importFrom(ggbeeswarm,geom_quasirandom)
importFrom(ggplot2,theme)
importFrom(ggprism,theme_prism)
importFrom(ggrepel,geom_label_repel)
importFrom(ggrepel,geom_text_repel)
importFrom(glue,glue_collapse)
importFrom(grDevices,col2rgb)
importFrom(grDevices,dev.off)
importFrom(grDevices,pdf)
importFrom(grDevices,rainbow)
importFrom(grid,gpar)
importFrom(grid,grid.circle)
importFrom(grid,grid.rect)
importFrom(janitor,adorn_totals)
importFrom(lifecycle,deprecate_soft)
importFrom(lifecycle,deprecate_stop)
importFrom(lifecycle,deprecate_warn)
importFrom(lifecycle,deprecated)
importFrom(lifecycle,is_present)
importFrom(magrittr,"%>%")
importFrom(methods,as)
importFrom(methods,hasArg)
importFrom(methods,new)
importFrom(methods,slot)
importFrom(paletteer,paletteer_c)
importFrom(paletteer,paletteer_d)
importFrom(patchwork,plot_annotation)
importFrom(patchwork,plot_layout)
importFrom(patchwork,wrap_plots)
importFrom(pbapply,pblapply)
importFrom(pbapply,pboptions)
importFrom(purrr,compact)
importFrom(purrr,discard)
importFrom(purrr,keep)
importFrom(purrr,map)
importFrom(purrr,map2)
importFrom(purrr,map_dbl)
importFrom(purrr,reduce)
importFrom(rlang,"!!")
importFrom(rlang,"%||%")
importFrom(rlang,":=")
importFrom(rlang,check_installed)
importFrom(rlang,is_installed)
importFrom(rlang,sym)
importFrom(scales,alpha)
importFrom(scales,hue_pal)
importFrom(scales,label_percent)
importFrom(scattermore,geom_scattermore)
importFrom(stats,as.dist)
importFrom(stats,complete.cases)
importFrom(stats,cor)
importFrom(stats,cutree)
importFrom(stats,density)
importFrom(stats,hclust)
importFrom(stats,kmeans)
importFrom(stats,lm)
importFrom(stats,mad)
importFrom(stats,median)
importFrom(stats,predict)
importFrom(stats,quantile)
importFrom(stats,setNames)
importFrom(stats,smooth.spline)
importFrom(stats,var)
importFrom(stringi,stri_replace_first_fixed)
importFrom(stringi,stri_replace_last_fixed)
importFrom(stringr,str_c)
importFrom(stringr,str_detect)
importFrom(stringr,str_extract)
importFrom(stringr,str_pad)
importFrom(stringr,str_replace)
importFrom(stringr,str_replace_na)
importFrom(stringr,str_to_lower)
importFrom(stringr,str_to_sentence)
importFrom(stringr,str_to_upper)
importFrom(stringr,str_wrap)
importFrom(tibble,column_to_rownames)
importFrom(tibble,rownames_to_column)
importFrom(tibble,tribble)
importFrom(tidyr,complete)
importFrom(tidyr,drop_na)
importFrom(tidyr,everything)
importFrom(tidyr,pivot_longer)
importFrom(tidyr,pivot_wider)
importFrom(tidyr,replace_na)
importFrom(tidyr,separate_wider_delim)
importFrom(utils,capture.output)
importFrom(utils,combn)
importFrom(utils,packageVersion)
importFrom(utils,read.csv)
importFrom(utils,read.delim)
importFrom(utils,read.table)
importFrom(utils,setTxtProgressBar)
importFrom(utils,txtProgressBar)
importFrom(utils,write.csv)
