% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Utilities.R
\name{Copy_From_GCP}
\alias{Copy_From_GCP}
\title{Copy folder from GCP bucket from R Console}
\usage{
Copy_From_GCP(folder_file_path, gcp_bucket_path)
}
\arguments{
\item{folder_file_path}{folder to be copied to GCP bucket.}

\item{gcp_bucket_path}{GCP bucket path to copy to files.}
}
\value{
No return value.  Performs system copy from GCP bucket.
}
\description{
Run command from R console without moving to terminal to copy folder from GCP bucket to local storage
}
\examples{
\dontrun{
Copy_From_GCP(folder_file_path = "plots/", gcp_bucket_path = "gs://bucket_name_and_folder_path")
}

}
\concept{organization_util}
